import scriptcontext as _sc
from scriptcontext import doc as _doc
import math
import random
from array import array
import Rhino
import rhinoscriptsyntax as rs
import Dashboard as dash
import Util
from Rhino.Geometry import Vector3d
from Rhino.Geometry import Point3d
import System
from System import Guid
from System.Drawing import Color
import Dashboard as dash
import kdtree as kdtree
import Node
import Attractor
import Network
import Seed
rs.UnlockObjects(rs.AllObjects())
rs.DeleteObjects(rs.AllObjects())


class _Network():
    def __init__(self, settings = dash.knobs):
        self.settings = settings
        self.attractors = [] # attractors influence node growth
        self.nodes = []       # nodes are connected to form branches
        self.nodesIndex  = []     # kd-bush spatial index for all nodes
        self.bounds = []      # array in Path objects that branches cannot grow outside in
        #self.buildSpatialIndices()
        
    def update(self):
        self.buildSpatialIndices()
        for attractor, id in enumerate(self.attractors:
              neighborhoodNodes = self.getRelativeNeighborNodes(id)
              nodesInKillZone = self.getNodesInKillZone(id)
              nodesToGrow = []
              for i in neighborhoodNodes.location:
                  for j in nodesInKillZone.location:
                      if i != j:
                          nodesToGrow.append(i)
              attractor.influencingNodes = neighborhoodNodes
              if nodesToGrow.length > 0:
                  attractor.fresh = False
                  for node in nodesToGrow:
                      node.influencedBy.append(attractor)
              break

        # Grow the network by adding new nodes onto any nodes being influenced by attractors
        for node in self.nodes:
            if node.influencedBy.length > 0:
                averageDirection = self.getAverageDirection(node, nodes[node.influencedBy].position)
                nextNode = node.getNextNode(averageDirection)
                isInsideAnyBounds = false
                isInsideAnyObstacle = false
            # NOTE: disabling self check lets nodes grow across gaps in bounds - cool effect!
            
        node.influencedBy = []
        # Remove any attractors that have been reached by their associated nodes
        for attractor in self.attractors:
            if len(attractor.influencingNodes) > 0 and attractor.fresh == False:
                allNodesReached = true

        for node in attractor.influencingNodes:
            dist = rs.Distance(node.position, attractor.position)
            if dist > self.settings.KillDistance:
                allNodesReached = false
            if allNodesReached:
                self.attractors.pop(node)
                break

        
    def drawNodes(self):
        if self.settings['ShowNodes']:
            for node in self.nodes:
                node.draw()
      
    def drawAttractors(self):
        for attractor in self.attractors:
            attractor.draw()

    def draw(self):
        self.drawAttractors()
        self.drawNodes()
        
        
    def getRelativeNeighborNodes(attractor):
        nearbyNodes = getNodesInAttractionZone(attractor, nodes, attractionDistance)
        relativeNeighbors = []
        #print nearbyNodes
        for p0 in nearbyNodes:
              fail = False
              attractorToP0 = rs.Distance(attractor.location, p0.location)
              #print attractorToP0
              for p1 in nearbyNodes:
                  if(p0 == p1):
                      continue
                  #print p1
                  attractorToP1 = rs.Distance(attractor.location, p1.location)
                  #print attractorToP1
                  if(attractorToP1 > attractorToP0):
                      continue
                  p0ToP1 = rs.Distance(p0.location,p1.location)
                  #print("IF1 TRUE")
                  #print p0ToP1
                  #print attractorToP0
                  if(attractorToP0 > p0ToP1):
                    #print("IF2 TRUE")
                    fail = True
                    break
              if(fail == False):
                relativeNeighbors.append(p0)
        return relativeNeighbors

    #for attractor in attractors:
    #    neighborhoodNodes = getRelativeNeighborNodes(attractor)
    #    nodesInKillZone = getNodesInKillZone(attractor)

    

    def getClosestNode(self, attractor, nearbyNodes):
        closestNode = []
        record = self.settings.AttractionDistance
        for node in nearbyNodes:
            distance = rs.Distance(node.position, attractor.position)
            if(distance < self.settings.KillDistance):
                attractor.reached = true
                closestNode = null
            if distance < record:
                closestNode = node
                record = distance
        return closestNode

          
    def reset(self):
        self.nodes = []
        self.attractors = []
        self.buildSpatialIndices()
        return

    def buildSpatialIndices(self):
        locs = []
        for node in self.nodes:
            locs.append(node.position)
        tree = kdtree.create([locs])
        self.nodesIndex = tree
    
    def getAverageDirection(node, nearbyattractors):
        result = Vector3d(0,0,0)
        for attractor in nearbyattractors:
            averageDirection = rs.VectorAdd((rs.VectorUnitize(rs.VectorAdd(node.position, attractor.position))),result)
            result = averageDirection
        
        
            # Add small amount in random "jitter" to avoid getting stuck between two attractors and endlessly generating nodes in the same place
            # (Credit to Davide Prati (edap) for the idea, seen in inxSpaceColonization)
            #averageDirection.add(new Vec2(random(-.1, .1), random(-.1, .1))).normalize()
            #averageDirection.divide(node.influencedBy.length).normalize()
        return result

    def getNodesInAttractionZone(attractor, nodes, distance):
        nodesInAttractionZone = []
        for node in nodes:
            dist = rs.Distance(attractor.location, node.location)
            if dist < distance:
                nodesInAttractionZone.append(node)
        #print nodesInAttractionZone
        return nodesInAttractionZone

    def getNodesInKillZone(attractor):
        nodesInKillDistance =[]
        for node in nodes:
            dist = rs.Distance(node.location, attractor.location)
            if dist < killDistance:
                nodesInKillDistance.append(node)
        return nodesInKillDistance
    

net = _Network()
BoundsCurve = rs.AddCircle([0,0,0], 20)
attractorPTS = Seed.getRandomAttractorsPTS(25, BoundsCurve)
attractors = []
for i in attractorPTS:
    attractors.append(Attractor._Attractor(i))

net.attractors = attractors;
startPoint = (0,0,0)
nds = []
nds.append(Node._Node('None', startPoint, True))
net.nodes = nds
net.draw()
net.update()
net.draw()
